@@ -1,24 +1,20 @@
-from tests.conftest import PROTOCOL, load_test_data_file, IaRequestsMock, ia_call
-
-try:
-    import ujson as json
-except ImportError:
-    import json
-
 import responses
 
+from internetarchive.utils import json
+from tests.conftest import PROTOCOL, IaRequestsMock, ia_call, load_test_data_file
+
 
 def test_ia_search_itemlist(capsys):
     test_scrape_response = load_test_data_file('scrape_response.json')
 
     with responses.RequestsMock(assert_all_requests_are_fired=False) as rsps:
-        url1 = ('{0}//archive.org/services/search/v1/scrape'
+        url1 = (f'{PROTOCOL}//archive.org/services/search/v1/scrape'
                 '?q=collection%3Aattentionkmartshoppers'
-                '&count=10000'.format(PROTOCOL))
-        url2 = ('{0}//archive.org/services/search/v1/scrape?'
+                '&count=10000')
+        url2 = (f'{PROTOCOL}//archive.org/services/search/v1/scrape?'
                 'cursor=W3siaWRlbnRpZmllciI6IjE5NjEtTC0wNTkxNCJ9XQ%3D%3D'
                 '&q=collection%3Aattentionkmartshoppers'
-                '&count=10000'.format(PROTOCOL))
+                '&count=10000')
         rsps.add(responses.POST, url1,
                  body=test_scrape_response,
                  match_querystring=True)
@@ -31,14 +27,14 @@ def test_ia_search_itemlist(capsys):
         ia_call(['ia', 'search', 'collection:attentionkmartshoppers', '--itemlist'])
 
     out, err = capsys.readouterr()
-    assert len(out.split()) == 200
+    assert len(set(out.split())) == 100
 
 
 def test_ia_search_num_found(capsys):
     with IaRequestsMock(assert_all_requests_are_fired=False) as rsps:
-        url = ('{0}//archive.org/services/search/v1/scrape'
+        url = (f'{PROTOCOL}//archive.org/services/search/v1/scrape'
                '?q=collection%3Anasa&total_only=true'
-               '&count=10000'.format(PROTOCOL))
+               '&count=10000')
         rsps.add(responses.POST, url,
                  body='{"items":[],"count":0,"total":50}',
                  match_querystring=True)
