@@ -1,28 +1,33 @@
-# -*- coding: utf-8 -*-
-from __future__ import unicode_literals
-
-import json
 import os
 import re
 
 import responses
-import six
-from requests.packages import urllib3
+import urllib3
 
-from internetarchive import get_session, get_item, get_files, modify_metadata, upload, \
-    download, search_items
-
-from internetarchive.utils import InvalidIdentifierException
-from tests.conftest import NASA_METADATA_PATH, PROTOCOL, IaRequestsMock, \
-    load_test_data_file, load_file
-
-PY3 = six.PY3
+from internetarchive import (
+    download,
+    get_files,
+    get_item,
+    get_session,
+    modify_metadata,
+    search_items,
+    upload,
+)
+from internetarchive.utils import InvalidIdentifierException, json
+from tests.conftest import (
+    NASA_METADATA_PATH,
+    PROTOCOL,
+    IaRequestsMock,
+    load_file,
+    load_test_data_file,
+)
 
 TEST_SEARCH_RESPONSE = load_test_data_file('advanced_search_response.json')
 TEST_SCRAPE_RESPONSE = load_test_data_file('scrape_response.json')
 _j = json.loads(TEST_SCRAPE_RESPONSE)
 del _j['cursor']
 _j['items'] = [{'identifier': 'nasa'}]
+_j['total'] = 1
 TEST_SCRAPE_RESPONSE = json.dumps(_j)
 
 
@@ -33,7 +38,7 @@ def test_get_session_with_config():
 
 def test_get_session_with_config_file(tmpdir):
     tmpdir.chdir()
-    test_conf = """[s3]\naccess = key2"""
+    test_conf = '[s3]\naccess = key2'
     with open('ia_test.ini', 'w') as fh:
         fh.write(test_conf)
     s = get_session(config_file='ia_test.ini')
@@ -52,7 +57,7 @@ def test_get_item_with_config(nasa_mocker):
 
 def test_get_item_with_config_file(tmpdir, nasa_mocker):
     tmpdir.chdir()
-    test_conf = """[s3]\naccess = key2"""
+    test_conf = '[s3]\naccess = key2'
     with open('ia_test.ini', 'w') as fh:
         fh.write(test_conf)
 
@@ -70,7 +75,7 @@ def test_get_item_with_kwargs():
     with IaRequestsMock(assert_all_requests_are_fired=False) as rsps:
         rsps.add_metadata_mock('nasa')
         item = get_item('nasa', http_adapter_kwargs={'max_retries': 13})
-        assert isinstance(item.session.adapters['{0}//'.format(PROTOCOL)].max_retries,
+        assert isinstance(item.session.adapters[f'{PROTOCOL}//'].max_retries,
                           urllib3.Retry)
 
     try:
@@ -83,15 +88,15 @@ def test_get_files():
     with IaRequestsMock(assert_all_requests_are_fired=False) as rsps:
         rsps.add_metadata_mock('nasa')
         files = get_files('nasa')
-        expected_files = set([
+        expected_files = {
             'NASAarchiveLogo.jpg',
             'globe_west_540.jpg',
             'nasa_reviews.xml',
             'nasa_meta.xml',
             'nasa_archive.torrent',
             'nasa_files.xml',
-        ])
-        assert set([f.name for f in files]) == expected_files
+        }
+        assert {f.name for f in files} == expected_files
 
 
 def test_get_files_with_get_item_kwargs(tmpdir):
@@ -111,7 +116,7 @@ def test_get_files_with_get_item_kwargs(tmpdir):
         assert len(files) == 1
         assert files[0].name == 'nasa_meta.xml'
 
-        test_conf = """[s3]\naccess = key2"""
+        test_conf = '[s3]\naccess = key2'
         with open('ia_test.ini', 'w') as fh:
             fh.write(test_conf)
         files = get_files('nasa', files='nasa_meta.xml',
@@ -155,11 +160,11 @@ def test_get_files_formats():
         assert files[0].name == 'globe_west_540.jpg'
 
         files = get_files('nasa', formats=['JPEG', 'Collection Header'])
-        expected_files = set([
+        expected_files = {
             'globe_west_540.jpg',
             'NASAarchiveLogo.jpg',
-        ])
-        assert set([f.name for f in files]) == expected_files
+        }
+        assert {f.name for f in files} == expected_files
 
 
 def test_get_files_glob_pattern():
@@ -171,22 +176,22 @@ def test_get_files_glob_pattern():
         assert files[0].name == 'nasa_archive.torrent'
 
         files = get_files('nasa', glob_pattern='*torrent|*jpg')
-        expected_files = set([
+        expected_files = {
             'globe_west_540.jpg',
             'NASAarchiveLogo.jpg',
             'nasa_archive.torrent',
-        ])
-        assert set([f.name for f in files]) == expected_files
+        }
+        assert {f.name for f in files} == expected_files
 
 
 def test_modify_metadata():
     with IaRequestsMock(assert_all_requests_are_fired=False) as rsps:
-        rsps.add(responses.GET, '{0}//archive.org/metadata/nasa'.format(PROTOCOL),
+        rsps.add(responses.GET, f'{PROTOCOL}//archive.org/metadata/nasa',
                  body='{"metadata":{"title":"foo"}}')
-        rsps.add(responses.POST, '{0}//archive.org/metadata/nasa'.format(PROTOCOL),
+        rsps.add(responses.POST, f'{PROTOCOL}//archive.org/metadata/nasa',
                  body=('{"success":true,"task_id":423444944,'
                        '"log":"https://catalogd.archive.org/log/423444944"}'))
-        r = modify_metadata('nasa', dict(foo=1))
+        r = modify_metadata('nasa', {'foo': 1})
         assert r.status_code == 200
         assert r.json() == {
             'task_id': 423444944,
@@ -208,14 +213,14 @@ def test_upload():
         rsps.add(responses.PUT, re.compile(r'.*s3.us.archive.org/.*'),
                  adding_headers=expected_s3_headers)
         rsps.add_metadata_mock('nasa')
-        rsps.add(responses.GET, '{0}//archive.org/metadata/nasa'.format(PROTOCOL),
+        rsps.add(responses.GET, f'{PROTOCOL}//archive.org/metadata/nasa',
                  body='{}')
         _responses = upload('nasa', NASA_METADATA_PATH,
                             access_key='test_access',
                             secret_key='test_secret')
         for response in _responses:
             req = response.request
-            headers = dict((k.lower(), str(v)) for k, v in req.headers.items())
+            headers = {k.lower(): str(v) for k, v in req.headers.items()}
             scanner_header = '%20'.join(
                 response.headers['x-archive-meta00-scanner'].split('%20')[:4])
             headers['x-archive-meta00-scanner'] = scanner_header
@@ -225,7 +230,7 @@ def test_upload():
             del headers['connection']
             del headers['user-agent']
             assert headers == expected_s3_headers
-            assert req.url == '{0}//s3.us.archive.org/nasa/nasa.json'.format(PROTOCOL)
+            assert req.url == f'{PROTOCOL}//s3.us.archive.org/nasa/nasa.json'
 
 
 def test_upload_validate_identifier():
@@ -234,7 +239,7 @@ def test_upload_validate_identifier():
               access_key='test_access',
               secret_key='test_secret',
               validate_identifier=True)
-        assert False
+        raise AssertionError("Given invalid identifier was not correctly validated.")
     except Exception as exc:
         assert isinstance(exc, InvalidIdentifierException)
 
@@ -250,7 +255,7 @@ def test_upload_validate_identifier():
         rsps.add(responses.PUT, re.compile(r'.*s3.us.archive.org/.*'),
                  adding_headers=expected_s3_headers)
         rsps.add_metadata_mock('nasa')
-        rsps.add(responses.GET, '{0}//archive.org/metadata/nasa'.format(PROTOCOL),
+        rsps.add(responses.GET, f'{PROTOCOL}//archive.org/metadata/nasa',
                  body='{}')
         upload('nasa', NASA_METADATA_PATH,
               access_key='test_access',
@@ -263,7 +268,7 @@ def test_download(tmpdir):
     tmpdir.chdir()
     with IaRequestsMock() as rsps:
         rsps.add(responses.GET,
-                 '{0}//archive.org/download/nasa/nasa_meta.xml'.format(PROTOCOL),
+                 f'{PROTOCOL}//archive.org/download/nasa/nasa_meta.xml',
                  body='test content')
         rsps.add_metadata_mock('nasa')
         download('nasa', 'nasa_meta.xml')
@@ -273,11 +278,11 @@ def test_download(tmpdir):
 
 
 def test_search_items(session):
-    results_url = ('{0}//archive.org/services/search/v1/scrape'
-                   '?q=identifier%3Anasa&count=10000'.format(PROTOCOL))
-    count_url = ('{0}//archive.org/services/search/v1/scrape'
+    results_url = (f'{PROTOCOL}//archive.org/services/search/v1/scrape'
+                   '?q=identifier%3Anasa&count=10000')
+    count_url = (f'{PROTOCOL}//archive.org/services/search/v1/scrape'
                  '?q=identifier%3Anasa&total_only=true'
-                 '&count=10000'.format(PROTOCOL))
+                 '&count=10000')
     with IaRequestsMock(assert_all_requests_are_fired=False) as rsps:
         rsps.add(responses.POST, results_url,
                  body=TEST_SCRAPE_RESPONSE,
@@ -294,6 +299,7 @@ def test_search_items(session):
         assert len(r.iter_as_results()) == 1
         assert list(r) == expected_results
         assert list(r.iter_as_results()) == expected_results
+        assert r.fts == False
 
 
 def test_search_items_with_fields(session):
@@ -302,12 +308,12 @@ def test_search_items_with_fields(session):
         {'identifier': 'nasa', 'title': 'NASA Images'}
     ]
     search_response_str = json.dumps(_j)
-    results_url = ('{0}//archive.org/services/search/v1/scrape'
+    results_url = (f'{PROTOCOL}//archive.org/services/search/v1/scrape'
                    '?q=identifier%3Anasa&count=10000'
-                   '&fields=identifier%2Ctitle'.format(PROTOCOL))
-    count_url = ('{0}//archive.org/services/search/v1/scrape'
+                   '&fields=identifier%2Ctitle')
+    count_url = (f'{PROTOCOL}//archive.org/services/search/v1/scrape'
                  '?q=identifier%3Anasa&total_only=true'
-                 '&count=10000'.format(PROTOCOL))
+                 '&count=10000')
     with IaRequestsMock() as rsps:
         rsps.add(responses.POST, results_url,
                  match_querystring=True,
@@ -324,7 +330,7 @@ def test_search_items_with_fields(session):
 def test_search_items_as_items(session):
     with IaRequestsMock(assert_all_requests_are_fired=False) as rsps:
         rsps.add(responses.POST,
-                 '{0}//archive.org/services/search/v1/scrape'.format(PROTOCOL),
+                 f'{PROTOCOL}//archive.org/services/search/v1/scrape',
                  body=TEST_SCRAPE_RESPONSE)
         rsps.add_metadata_mock('nasa')
         r = search_items('identifier:nasa', archive_session=session)
@@ -332,17 +338,50 @@ def test_search_items_as_items(session):
         assert r.iter_as_items().search == r
 
 
+def test_search_items_fts(session):
+    with IaRequestsMock(assert_all_requests_are_fired=False) as rsps:
+        rsps.add(responses.POST,
+                 f'{PROTOCOL}//be-api.us.archive.org/ia-pub-fts-api',
+                 body=TEST_SCRAPE_RESPONSE)
+        rsps.add_metadata_mock('nasa')
+
+        r = search_items('nina simone',
+                         full_text_search=True,
+                         archive_session=session)
+        print(r.search_url)
+        assert r.fts == True
+        assert r.dsl_fts == False
+        assert r.query == '!L nina simone'
+        assert r.params == {'count': 10000, 'q': '!L nina simone'}
+
+        r = search_items('nina simone',
+                         full_text_search=True,
+                         dsl_fts=True,
+                         archive_session=session)
+        assert r.fts == True
+        assert r.dsl_fts == True
+        assert r.query == 'nina simone'
+        assert r.params == {'count': 10000, 'q': 'nina simone'}
+        r = search_items('nina simone',
+                         dsl_fts=True,
+                         archive_session=session)
+        assert r.fts == True
+        assert r.dsl_fts == True
+        assert r.query == 'nina simone'
+        assert r.params == {'count': 10000, 'q': 'nina simone'}
+
+
 def test_page_row_specification(session):
     _j = json.loads(TEST_SEARCH_RESPONSE)
     _j['response']['items'] = [{'identifier': 'nasa'}]
     _j['response']['numFound'] = 1
     _search_r = json.dumps(_j)
     with IaRequestsMock(assert_all_requests_are_fired=False) as rsps:
-        rsps.add(responses.GET, '{0}//archive.org/advancedsearch.php'.format(PROTOCOL),
+        rsps.add(responses.GET, f'{PROTOCOL}//archive.org/advancedsearch.php',
                  body=_search_r)
         rsps.add_metadata_mock('nasa')
         rsps.add(responses.POST,
-                 '{0}//archive.org/services/search/v1/scrape'.format(PROTOCOL),
+                 f'{PROTOCOL}//archive.org/services/search/v1/scrape',
                  body='{"items":[],"count":0,"total":1}',
                  match_querystring=False,
                  content_type='application/json; charset=UTF-8')
